"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __exportStar = (this && this.__exportStar) || function(m, exports) {
    for (var p in m) if (p !== "default" && !exports.hasOwnProperty(p)) __createBinding(exports, m, p);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseProcess = void 0;
const child_process_1 = require("child_process");
const dayjs_1 = __importDefault(require("dayjs"));
const lcovConcat_1 = require("../lcovConcat");
const parsers_1 = require("../../parsers");
const streams_1 = require("../../streams");
class BaseProcess {
    constructor(lcovPath, opts = { stream: {} }) {
        this.opts = {
            stream: {},
        };
        this.lcov = { $: { linesCovered: 0, linesValid: 0 }, detail: {} };
        this.firstGitMessage = {};
        this.formatData = {
            total: { increLine: 0, covLine: 0, increRate: '' },
            files: [],
        };
        if (typeof lcovPath === 'string') {
            this.lcovPath = [lcovPath];
        }
        else {
            this.lcovPath = lcovPath;
        }
        const startDay = dayjs_1.default().startOf('month').format('YYYY-MM-DD');
        this.opts.cwd = opts.cwd || process.cwd();
        this.opts.since = opts.since || startDay;
        this.opts.stream.name = opts.stream.name || 'file';
        this.opts.stream.opts = opts.stream.opts;
        this.opts.output = opts.output;
        this.opts.hash = opts.hash;
    }
    exec() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.opts.hash) {
                this.getInfoByHash(this.opts.hash);
            }
            else {
                yield this.getLog();
            }
            this.createInfo();
            yield this.getLcov();
            this.format();
            if (this.opts.output) {
                this.output();
            }
            return {
                data: this.formatData,
                commit: this.firstGitMessage,
                createInfo: this.firstInfo,
            };
        });
    }
    getLcov() {
        return __awaiter(this, void 0, void 0, function* () {
            this.lcov = yield lcovConcat_1.increaseLcovConcat(this.lcovPath, {
                cwd: this.opts.cwd,
                hash: this.firstGitMessage.hash,
            });
        });
    }
    format() {
        this.formatData.total.covLine = this.lcov.$.linesCovered;
        this.formatData.total.increLine = this.lcov.$.linesCovered;
        this.formatData.total.increRate = `${((this.lcov.$.linesCovered / this.lcov.$.linesValid) *
            100).toFixed(2)}%`;
        Object.entries(this.lcov.detail).forEach(([name, detail]) => {
            const temp = {
                name,
                increLine: detail.linesValid,
                covLine: detail.linesCovered,
                increRate: `${(detail.lineRate * 100).toFixed(2)}%`,
                detail: [],
            };
            temp.detail = detail.lines.map((item) => {
                return {
                    number: +item.number,
                    hits: item.hits,
                };
            });
            this.formatData.files.push(temp);
        });
    }
    getLog() {
        return __awaiter(this, void 0, void 0, function* () {
            const subDate = dayjs_1.default(this.opts.since).subtract(1, 'day').format('YYYY-MM-DD');
            const res = yield new parsers_1.LogParser({
                repo: this.opts.cwd,
                until: subDate,
            }).run();
            [this.firstGitMessage] = res;
        });
    }
    output() {
        var _a, _b;
        let tempStream;
        if (((_a = this.opts.stream) === null || _a === void 0 ? void 0 : _a.name) === 'file') {
            tempStream = new streams_1.FileStream({
                data: this.formatData,
                commit: this.firstGitMessage,
                createInfo: this.firstInfo,
            }, this.opts.stream.opts);
        }
        else if (((_b = this.opts.stream) === null || _b === void 0 ? void 0 : _b.name) === 'stdio') {
            tempStream = new streams_1.StdoutStream({
                data: this.formatData,
                commit: this.firstGitMessage,
                createInfo: this.firstInfo,
            }, this.opts.stream.opts);
        }
        else {
            throw new Error('参数错误');
        }
        tempStream.outToStream();
    }
    createInfo() {
        const res = child_process_1.execSync('git log --reverse --pretty="%H!!!%h!!!%aN!!!%aE!!!%ad!!!%B"', {
            cwd: this.opts.cwd,
        })
            .toString()
            .split('\n');
        const first = res[0].split('!!!');
        this.firstInfo = {
            hash: first[0],
            abbrevHash: first[1],
            authorName: first[2],
            authorEmail: first[3],
            authorDate: first[4],
            subject: first[5],
        };
    }
    getInfoByHash(hash) {
        const res = child_process_1.execSync(`git log ${hash} --pretty="%H!!!%h!!!%aN!!!%aE!!!%ad!!!%B"`, {
            cwd: this.opts.cwd,
        })
            .toString()
            .split('\n');
        const first = res[0].split('!!!');
        this.firstGitMessage = {
            hash: first[0],
            abbrevHash: first[1],
            authorName: first[2],
            authorDate: first[4],
            subject: first[5],
        };
    }
}
exports.BaseProcess = BaseProcess;
__exportStar(require("./async"), exports);
