"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.IncreaseProcess = void 0;
const dayjs_1 = __importDefault(require("dayjs"));
const child_process_1 = require("child_process");
const index_1 = require("./index");
const concat_1 = require("../../concat");
const parsers_1 = require("../../parsers");
const readLcov_1 = require("../../utils/readLcov");
class IncreaseProcess extends index_1.BaseProcess {
    constructor(lcovPath, opts = { stream: {} }) {
        super();
        this.opts = {
            stream: {},
        };
        this.firstGitMessage = {};
        if (typeof lcovPath === 'string') {
            this.lcovPath = [lcovPath];
        }
        else {
            this.lcovPath = lcovPath;
        }
        const startDay = dayjs_1.default().startOf('month').format('YYYY-MM-DD');
        this.opts.cwd = opts.cwd || process.cwd();
        this.opts.since = opts.since || startDay;
        this.opts.stream.name = opts.stream.name || 'file';
        this.opts.stream.opts = opts.stream.opts;
        this.opts.output = opts.output;
        this.opts.hash = opts.hash;
    }
    exec() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.opts.hash) {
                this.getInfoByHash(this.opts.hash);
            }
            else {
                yield this.getLog();
            }
            this.createInfo();
            yield this.getLcov();
            this.format();
            if (this.opts.output) {
                this.output({
                    data: this.formatData,
                    commit: this.firstGitMessage,
                    createInfo: this.firstInfo,
                });
            }
            return {
                data: this.formatData,
                commit: this.firstGitMessage,
                createInfo: this.firstInfo,
            };
        });
    }
    getLcov() {
        return __awaiter(this, void 0, void 0, function* () {
            const res = yield readLcov_1.getLcovFile(this.lcovPath);
            this.lcov = (yield new concat_1.IncreaseConcat({
                cwd: this.opts.cwd,
                hash: this.firstGitMessage.hash,
            }).concat(...res)).getRes();
        });
    }
    getLog() {
        return __awaiter(this, void 0, void 0, function* () {
            const subDate = dayjs_1.default(this.opts.since).subtract(1, 'day').format('YYYY-MM-DD');
            const res = yield new parsers_1.LogParser({
                repo: this.opts.cwd,
                until: subDate,
            }).run();
            [this.firstGitMessage] = res;
        });
    }
    getInfoByHash(hash) {
        const res = child_process_1.execSync(`git log ${hash} --pretty="%H!!!%h!!!%aN!!!%aE!!!%ad!!!%B"`, {
            cwd: this.opts.cwd,
        })
            .toString()
            .split('\n');
        const first = res[0].split('!!!');
        this.firstGitMessage = {
            hash: first[0],
            abbrevHash: first[1],
            authorName: first[2],
            authorDate: first[4],
            subject: first[5],
        };
    }
}
exports.IncreaseProcess = IncreaseProcess;
